@@ -1,4 +1,4 @@
 
 from ._glpk import glpk
-from ._utils import mpsread, mpswrite
+from ._fileio import mpsread, mpswrite, lpwrite
 from ._glpk_defines import GLPK
