@@ -2,13 +2,15 @@
 
 import ctypes
 from warnings import warn
-import platform
 
 import numpy as np
 from scipy.sparse import coo_matrix
 from scipy.optimize import OptimizeWarning, OptimizeResult
+from scipy.optimize._linprog_util import _LPProblem, _clean_inputs
 
 from ._glpk_defines import GLPK, glp_smcp, glp_iptcp, glp_bfcp, glp_iocp
+from ._utils import _fill_prob
+
 
 def glpk(
         c,
@@ -27,7 +29,6 @@ def glpk(
         simplex_options=None,
         ip_options=None,
         mip_options=None,
-        libpath='libglpk.%s' % ('dll' if platform.system() == 'Windows' else 'so'),
 ):
     '''GLPK ctypes interface.
 
@@ -41,15 +42,13 @@ def glpk(
     A_eq : 2-D array (k, n)
         scipy.sparse.coo_matrix
     b_eq : 1-D array (k,)
-    bounds : list (n,) of tuple (3,)
+    bounds : None or list (n,) of tuple (2,) or tuple (2,)
         The jth entry in the list corresponds to the jth objective coefficient.
-        Each entry is made up of a tuple describing the bounds:
-
-            - ``type`` : {GLP_FR, GLP_LO, GLP_UP, GLP_DB, GLP_FX}
-            - ``lb`` : double
-            - ``up`` : double
-
-        If the entry is ``None``, then ``type=GLP_FX`` and ``ub==lb==0``.
+        Each entry is made up of a tuple describing the bounds.
+        Use None to indicate that there is no bound. By default, bounds are
+        (0, None) (all decision variables are non-negative). If a single tuple
+        (min, max) is provided, then min and max will serve as bounds for all
+        decision variables.
     solver : { 'simplex', 'interior', 'mip' }
         Use simplex (LP/MIP) or interior point method (LP only).
         Default is ``simplex``.
@@ -119,13 +118,38 @@ def glpk(
                     - ``norelax`` : standard "textbook" ratio test
                     - ``flip`` : long-step ratio test
 
+            - tol_bnd : double
+                Tolerance used to check if the basic solution is primal
+                feasible. (Default: 1e-7).
+
+            - tol_dj : double
+                Tolerance used to check if the basic solution is dual
+                feasible. (Default: 1e-7).
+
+            - tol_piv : double
+                Tolerance used to choose eligble pivotal elements of
+                the simplex table. (Default: 1e-10).
+
+            - obj_ll : double
+                Lower limit of the objective function. If the objective
+                function reaches this limit and continues decreasing,
+                the solver terminates the search. Used in the dual simplex
+                only. (Default: -DBL_MAX -- the largest finite float64).
+
+            - obj_ul : double
+                Upper limit of the objective function. If the objective
+                function reaches this limit and continues increasing,
+                the solver terminates the search. Used in the dual simplex
+                only. (Default: +DBL_MAX -- the largest finite float64).
+
             - presolve : bool
                 Use presolver (assumes ``scale=True`` and
                 ``init_basis='adv'``. Default is ``True``.
 
             - exact : bool
                 Use simplex method based on exact arithmetic.
-                Default is ``False``.
+                Default is ``False``. If ``True``, all other
+                ``simplex_option`` fields are ignored.
 
     ip_options : dict
         Options specific to interior-pooint solver.
@@ -139,7 +163,8 @@ def glpk(
                     - ``qmd`` : quotient minimum degree ordering
                     - ``amd`` : approximate minimum degree ordering
                     - ``symamd`` : approximate minimum degree ordering
-                        algorithm for Cholesky factorization of symmetric matrices.
+                        algorithm for Cholesky factorization of symmetric
+                        matrices.
 
     mip_options : dict
         Options specific to MIP solver.
@@ -165,8 +190,8 @@ def glpk(
                     - ``last`` : branch on last integer variable
                     - ``mostf`` : branch on most fractional variable
                     - ``drtom`` : branch using heuristic by Driebeck and Tomlin
-                    - ``pcost`` : branch using hybrid pseudocost heuristic (may be
-                        useful for hard instances)
+                    - ``pcost`` : branch using hybrid pseudocost heuristic
+                                  (may be useful for hard instances)
 
             - backtrack : { 'dfs', 'bfs', 'bestp', 'bestb' }
                 Backtracking rule. Default is ``bestb``.
@@ -210,30 +235,34 @@ def glpk(
                     - ``clique`` : clique cuts
                     - ``all`` : generate all cuts above
 
-            - gap_tol : float
-                Relative mip gap tolerance.
-
+            - tol_int : float
+                Absolute tolerance used to check if optimal solution to the
+                current LP relaxation is integer feasible.
+                (Default: 1e-5).
+            - tol_obj : float
+                Relative tolerance used to check if the objective value in
+                optimal solution to the current LP relaxation is not better
+                than in the best known integer feasible solution.
+                (Default: 1e-7).
+            - mip_gap : float
+                Relative mip gap tolerance. If the relative mip gap for
+                currently known best integer feasiblesolution falls below
+                this tolerance, the solver terminates the search. This allows
+                obtaining suboptimal integer feasible solutions if solving the
+                problem to optimality takes too long time.
+                (Default: 0.0).
             - bound : float
                 add inequality obj <= bound (minimization) or
                 obj >= bound (maximization) to integer feasibility
                 problem (assumes ``minisat=True``).
 
+    Notes
+    -----
+    In general, don't change tolerances without a detailed understanding
+    of their purposes.
     '''
 
     # Housekeeping
-    if bounds is None:
-        # Default bound is [0, inf)
-        bounds = [(GLPK.GLP_LO, 0, 0)]*len(c)
-    if A_ub is None:
-        # we need numpy arrays
-        A_ub = np.empty((0, len(c)))
-    if b_ub is None:
-        # just need something iterable
-        b_ub = []
-    if A_eq is None:
-        A_eq = np.empty((0, len(c)))
-    if b_eq is None:
-        b_eq = []
     if simplex_options is None:
         simplex_options = {}
     if ip_options is None:
@@ -241,58 +270,12 @@ def glpk(
     if mip_options is None:
         mip_options = {}
 
+    # Create and fill the GLPK problem struct
+    prob, lp = _fill_prob(c, A_ub, b_ub, A_eq, b_eq, bounds, sense, 'problem-name')
+    c, A_ub, b_ub, A_eq, b_eq, bounds, _x0 = lp
+
     # Get the library
-    _lib = GLPK(libpath)._lib
-
-    # Create problem instance
-    prob = _lib.glp_create_prob()
-
-    # Give problem a name
-    _lib.glp_set_prob_name(prob, b'problem-name')
-
-    # Set objective name
-    _lib.glp_set_obj_name(prob, b'obj-name')
-
-    # Set objective sense
-    _lib.glp_set_obj_dir(prob, sense)
-
-    # Set objective coefficients and column bounds
-    first_col = _lib.glp_add_cols(prob, len(c))
-    for ii, (c0, bnd) in enumerate(zip(c, bounds)):
-        _lib.glp_set_obj_coef(prob, ii + first_col, c0)
-        _lib.glp_set_col_name(prob, ii + first_col, b'c%d' % ii) # name is c[idx], idx is 0-based index
-
-        if bnd is not None:
-            _lib.glp_set_col_bnds(prob, ii + first_col, bnd[0], bnd[1], bnd[2])
-        # else: default is GLP_FX with lb=0, ub=0
-
-    # Need to load both matrices at the same time
-    A = coo_matrix(np.concatenate((A_ub, A_eq), axis=0)) # coo for (i, j, val) format
-    b = np.concatenate((b_ub, b_eq))
-    first_row = _lib.glp_add_rows(prob, A.shape[0])
-
-    # prepend an element and make 1-based index
-    # b/c GLPK expects indices starting at 1
-    nnz = A.nnz
-    rows = np.concatenate(([-1], A.row + first_row)).astype(ctypes.c_int)
-    cols = np.concatenate(([-1], A.col + first_col)).astype(ctypes.c_int)
-    values = np.concatenate(([0], A.data)).astype(ctypes.c_double)
-    _lib.glp_load_matrix(
-        prob,
-        nnz,
-        rows,
-        cols,
-        values,
-    )
-
-    # Set row bounds
-    # Upper bounds (b_ub):
-    for ii, b0 in enumerate(b_ub):
-        # lb is ignored for upper bounds
-        _lib.glp_set_row_bnds(prob, ii + first_row, GLPK.GLP_UP, 0, b0)
-    # Equalities (b_eq)
-    for ii, b0 in enumerate(b_eq):
-        _lib.glp_set_row_bnds(prob, ii + first_row + len(b_ub), GLPK.GLP_FX, b0, b0)
+    _lib = GLPK()._lib
 
     # Scale the problem
     _lib.glp_scale_prob(prob, GLPK.GLP_SF_AUTO) # do auto scaling for now
@@ -345,6 +328,13 @@ def glpk(
             'norelax': GLPK.GLP_RT_STD,
             'flip': GLPK.GLP_RT_FLIP,
         }[simplex_options.get('ratio', 'relax')]
+        smcp.tol_bnd = simplex_options.get('tol_bnd', 1e-7)
+        smcp.tol_dj = simplex_options.get('tol_dj', 1e-7)
+        smcp.tol_piv = simplex_options.get('tol_piv', 1e-10)
+        if simplex_options.get('obj_ll', False):
+            smcp.obj_ll = simplex_options['obj_ll']
+        if simplex_options.get('obj_ul', False):
+            smcp.obj_ul = simplex_options['obj_ul']
         smcp.it_lim = maxit
         smcp.tm_lim = timeout
         smcp.presolve = {
@@ -379,8 +369,8 @@ def glpk(
             res.x = np.array([_lib.glp_get_col_prim(prob, ii) for ii in range(1, len(c)+1)])
             res.dual = np.array([_lib.glp_get_col_dual(prob, ii) for ii in range(1, len(b_ub)+1)])
 
-            # We don't get slack without doing sensitivity analysis since GLPK uses
-            # auxiliary variables instead of slack!
+            # We don't get slack without doing sensitivity analysis since GLPK
+            # uses auxiliary variables instead of slack!
             res.slack = b_ub - A_ub @ res.x
             res.con = b_eq - A_eq @ res.x
 
@@ -498,7 +488,9 @@ def glpk(
         if 'clique' in cuts:
             iocp.clq_cuts = GLPK.GLP_ON
 
-        iocp.mip_gap = mip_options.get('gap_tol', 0.0)
+        iocp.tol_int = mip_options.get('tol_int', 1e-5)
+        iocp.tol_obj = mip_options.get('tol_obj', 1e-7)
+        iocp.mip_gap = mip_options.get('mip_gap', 0.0)
         iocp.tm_lim = timeout
         iocp.presolve = {
             True: GLPK.GLP_ON,
@@ -531,7 +523,6 @@ def glpk(
             res.fun = _lib.glp_mip_obj_val(prob)
             res.x = np.array([_lib.glp_mip_col_val(prob, ii) for ii in range(1, len(c)+1)])
 
-
     else:
         raise ValueError('"%s" is not a recognized solver.' % solver)
 
@@ -539,11 +530,12 @@ def glpk(
     _lib.glp_delete_prob(prob)
 
     # Map status codes to scipy:
-    #res.status = {
-    #    GLPK.GLP_OPT: 0,
-    #}[res.status]
+    # res.status = {
+    #     GLPK.GLP_OPT: 0,
+    # }[res.status]
 
     return res
 
+
 if __name__ == '__main__':
     pass
