@@ -1,7 +1,6 @@
 
 import ctypes
-import os
-import pathlib
+import importlib.util
 
 from numpy.ctypeslib import ndpointer
 
@@ -139,6 +138,10 @@ class glp_prob(ctypes.Structure):
 
 class GLPK:
 
+    # Where is GLPK? Find where the C extension module was installed:
+    _glpk_lib_path = importlib.util.find_spec("glpk4_65").origin
+
+
     INT_MAX = ctypes.c_uint(-1).value // 2
     GLP_ON = 1
     GLP_OFF = 0
@@ -283,15 +286,10 @@ class GLPK:
     GLP_IV = 2 # integer variable
     GLP_BV = 3 # binary variable
 
-    def __init__(self, libpath):
+    def __init__(self):
 
         # Load the shared library;
-        # Windows Octave installs require that we load from
-        # the same directory as libglpk.dll is found
-        cwd = os.getcwd()
-        os.chdir(pathlib.Path(libpath).parent)
-        _lib = ctypes.cdll.LoadLibrary(libpath)
-        os.chdir(cwd)
+        _lib = ctypes.cdll.LoadLibrary(self._glpk_lib_path)
 
         _lib.glp_version.restype = ctypes.c_char_p
 
@@ -530,6 +528,13 @@ class GLPK:
             ctypes.c_char_p,           # filename
         ]
 
+        _lib.glp_write_lp.restype = ctypes.c_int
+        _lib.glp_write_lp.argtypes = [
+            ctypes.POINTER(glp_prob),
+            ctypes.POINTER(glp_mpscp), # should be NULL
+            ctypes.c_char_p,           # filename
+        ]
+        
         # LP Basis Factorization
         _lib.glp_get_bfcp.restype = None
         _lib.glp_get_bfcp.argtypes = [ctypes.POINTER(glp_prob), ctypes.POINTER(glp_bfcp)]
